import sanic
import asyncio
import pytest
from sanic.app import Sanic
from sanic.websocket import WebSocketProtocol
from sanic import response


collect_ignore = []

if sanic.__version__ <= '0.6.0':
    collect_ignore.append("test_client_websocket.py")


# pytest_plugins = 'pytest_sanic.plugin'


@pytest.yield_fixture
def app():
    app = Sanic("test_sanic_app")

    @app.route("/test_get", methods=['GET'])
    async def test_get(request):
        return response.json({"GET": True})

    @app.route("/test_post", methods=['POST'])
    async def test_post(request):
        return response.json({"POST": True})

    @app.route("/test_put", methods=['PUT'])
    async def test_put(request):
        return response.json({"PUT": True})

    @app.route("/test_delete", methods=['DELETE'])
    async def test_delete(request):
        return response.json({"DELETE": True})

    @app.route("/test_patch", methods=['PATCH'])
    async def test_patch(request):
        return response.json({"PATCH": True})

    @app.route("/test_options", methods=['OPTIONS'])
    async def test_options(request):
        return response.json({"OPTIONS": True})

    @app.route("/test_head", methods=['HEAD'])
    async def test_head(request):
        return response.json({"HEAD": True})

    @app.websocket("/test_ws")
    async def test_ws(request, ws):
        data = await ws.recv()
        await ws.send(data)

    @app.listener('before_server_start')
    async def mock_init_db(app, loop):
        await asyncio.sleep(0.01)

    yield app


@pytest.fixture
def sanic_server(loop, app, test_server):
    return loop.run_until_complete(test_server(app))


@pytest.fixture
def test_cli(loop, app, test_client):
    return loop.run_until_complete(test_client(app, protocol=WebSocketProtocol))



import pytest


def test_fixture_unused_port(unused_port):
    assert unused_port > 1024 and unused_port < 65535


class MyEventLoop:

    def close(self):
        pass


@pytest.yield_fixture
def loop():
    loop = MyEventLoop()
    yield loop
    loop.close()


def test_loop_override(loop):
    assert isinstance(loop, MyEventLoop) is True



import asyncio
import pytest
import inspect
import socket
from .utils import TestServer, TestClient
try:
    import uvloop
except:  # pragma: no cover
    uvloop = None


LOOP_INIT = None
LOOP_KEY = 'loop'

def pytest_addoption(parser):
    parser.addoption(
        '--loop', default=None,
        help='run tests with specific loop: aioloop, uvloop')


def pytest_configure(config):
    global LOOP_INIT
    loop_name = config.getoption('--loop')
    factory = {
        "aioloop": asyncio.new_event_loop,
    }
    if uvloop is not None:
        factory["uvloop"] = uvloop.new_event_loop

    if loop_name:
        if loop_name not in factory:
            raise ValueError(
                "{name} is not valid option".format(name=loop_name)
            )
        LOOP_INIT = factory[loop_name]
    else:
        LOOP_INIT = factory["aioloop"]


@pytest.yield_fixture
def loop():
    """
    Default event loop, you should only use this event loop in your tests.
    """
    loop = LOOP_INIT()
    asyncio.set_event_loop(loop)
    yield loop
    loop.close()


def pytest_pycollect_makeitem(collector, name, obj):
    """
    pytest should also collect coroutines.
    """
    if collector.funcnamefilter(name) and _is_coroutine(obj):
        return list(collector._genfunctions(name, obj))


def pytest_pyfunc_call(pyfuncitem):
    """
    Run test coroutines in an event loop.
    """
    if _is_coroutine(pyfuncitem.function):
        loop = pyfuncitem.funcargs[LOOP_KEY]
        funcargs = pyfuncitem.funcargs
        testargs = {}
        for arg in pyfuncitem._fixtureinfo.argnames:
            testargs[arg] = funcargs[arg]
        loop.run_until_complete(
            loop.create_task(
                pyfuncitem.obj(**testargs)
            )
        )
        return True


def pytest_runtest_setup(item):
    """
    append a loop fixture to all test func.
    """
    if hasattr(item, 'fixturenames') and LOOP_KEY not in item.fixturenames:
        item.fixturenames.append(LOOP_KEY)


@pytest.fixture
def unused_port():
    with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as s:
        s.bind(('127.0.0.1', 0))
        return s.getsockname()[1]


@pytest.yield_fixture
def test_server(loop):
    """
    Create a TestServer instance based on a Sanic Application.

    test_server(app, **kwargs)
    """

    servers = []

    async def create_server(app, **kwargs):
        server = TestServer(app, loop=loop, **kwargs)
        await server.start_server()
        servers.append(server)
        return server

    yield create_server

    # Close Server
    if servers:
        for server in servers:
            loop.run_until_complete(server.close())


@pytest.yield_fixture
def test_client(loop):
    """
    Create a TestClient instance for test easy use.

    test_client(app, **kwargs)
    """
    clients = []

    async def create_client(app, **kwargs):
        client = TestClient(app, loop=loop, **kwargs)
        await client.start_server()
        clients.append(client)
        return client

    yield create_client

    # Clean up
    if clients:
        for client in clients:
            loop.run_until_complete(client.close())


# Helper Functions

def _is_coroutine(obj):
    return asyncio.iscoroutinefunction(obj) or inspect.isgeneratorfunction(obj)



#!/usr/bin/env python
import os
from setuptools import setup, find_packages

base = os.path.dirname(os.path.abspath(__file__))

README_PATH = os.path.join(base, "README.rst")

install_requires = [
    'pytest',
    'sanic',
    'aiohttp',
]

tests_require = []

setup(name='pytest-sanic',
      version='0.1.8',
      description='',
      long_description=open(README_PATH).read(),
      author='Yun Xu',
      author_email='yunxu1992@gmail.com',
      url='',
      packages=find_packages(),
      install_requires=install_requires,
      classifiers=[
          'Development Status :: 3 - Alpha',
          'Operating System :: MacOS',
          'Operating System :: POSIX :: Linux',
          'Topic :: System :: Software Distribution',
          'License :: OSI Approved :: MIT License',
          'Programming Language :: Python',
          'Programming Language :: Python :: 3',
          'Programming Language :: Python :: 3.5',
      ],
      entry_points={
        'pytest11': ['sanic = pytest_sanic.plugin'],
      },
)
